@@ -41,27 +41,27 @@ class ogs_base(bb_base):
                     packages=['virtualenv', 'pre-commit', 'cmake-format'])
         self += packages(ospackages=self.__ospackages,
                          apt_ppas=['ppa:git-core/ppa'],
-                         epel=True, powertools=True)
+                         epel=True)
         self += shell(commands=self.__commands)
-        self += environment(variables={'CMAKE_GENERATOR': 'Ninja'})
+        self += environment(variables={
+            'CMAKE_GENERATOR': 'Ninja',
+            'PATH': '/usr/local/poetry/bin:$PATH'
+        })
 
     def __setup(self):
-        self.__ospackages.extend(['git', 'git-lfs', 'ninja-build'])
+        self.__ospackages.extend(['git'])
 
         dist = 'deb'
         if hpccm.config.g_linux_distro == linux_distro.CENTOS:
             dist = 'rpm'
-        if hpccm.config.g_linux_distro == linux_distro.UBUNTU:
             self.__commands.extend([
-                "apt-get update",
-                # Workaround for https://github.com/git-lfs/git-lfs/issues/3474
-                "apt-get install -y dirmngr --install-recommends",
-                "apt-key adv --keyserver keyserver.ubuntu.com --recv-keys "
-                "6B05F25D762E3157"
+                'wget https://github.com/ninja-build/ninja/releases/download/v1.10.0/ninja-linux.zip',
+                'unzip ninja-linux.zip',
+                'mv ninja /usr/local/bin',
+                'rm ninja-linux.zip'
             ])
-        self.__commands.append(
-            "curl -s https://packagecloud.io/install/repositories/github/"
-            "git-lfs/script.{0}.sh | bash".format(dist))
+        else:
+            self.__ospackages.extend(['ninja-build'])
 
         if hpccm.config.g_ctype == hpccm.container_type.SINGULARITY:
             self.__ospackages.append('locales')
@@ -72,12 +72,17 @@ class ogs_base(bb_base):
                 'locale-gen en_US.UTF-8'
             ])
 
-        self.__commands.append('git lfs install')
-
         # Common directories
         self.__commands.append(
             'mkdir -p /apps /scratch /lustre /work /projects /data')
 
+        # Poetry
+        self.__ospackages.append('python3-venv')
+        self.__commands.append('''curl -sSL \
+               https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py \
+               | POETRY_HOME=/usr/local/poetry POETRY_VERSION=1.0.10 python3'''
+                               )
+
     def runtime(self, _from='0'):
         p = python(devel=True, python2=False)
         instructions = [
