@@ -21,7 +21,8 @@ from packaging import version
 import hpccm
 from hpccm import linux_distro
 from hpccm.building_blocks import packages, mlnx_ofed, knem, ucx, openmpi, \
-    boost, pip, scif, llvm, gnu, ofed, cmake, slurm_pmi2, pmix
+    boost, pip, scif, llvm, gnu, ofed, cmake, slurm_pmi2, pmix, generic_cmake, \
+    generic_autotools
 from hpccm.primitives import baseimage, comment, user, environment, raw, \
     label, shell, copy
 
@@ -30,17 +31,14 @@ from ogscm.cli_args import Cli_Args
 from ogscm.config import package_manager
 from ogscm.container_info import container_info
 from ogscm.version import __version__
-from ogscm.building_blocks import ccache, cppcheck, cvode, eigen, iwyy, \
-    jenkins_node, ogs_base, ogs, osu_benchmarks, petsc, pm_conan, vtk, \
-    pm_spack, paraview
+from ogscm.building_blocks import ccache, ogs_base, ogs, osu_benchmarks, \
+    pm_conan, paraview
 
 
 def main():  # pragma: no cover
     cli = Cli_Args()
     args = cli.parse_args()
 
-    if args.jenkins or args.gitlab:
-        args.ogs = ['off']
     if args.deploy != '':
         args.build = True
         args.convert = True
@@ -65,14 +63,13 @@ def main():  # pragma: no cover
                 '--file can only be used when generating a single image definition'
             )
             quit(1)
-        if (len(c) > 1 and args.sif_file != '') or (args.sif_file != '' and args.convert == False):
-            print(
-                '--sif_file can only be used when generating a single image '
-                'definition and --convert is given'
-            )
+        if (len(c) > 1 and args.sif_file != '') or (args.sif_file != ''
+                                                    and args.convert == False):
+            print('--sif_file can only be used when generating a single image '
+                  'definition and --convert is given')
             quit(1)
-        if ogs_version == 'off' and len(
-                cmake_args) > 0 and cmake_args[0] != '':
+        if (ogs_version == 'off' or ogs_version
+                == 'clean') and len(cmake_args) > 0 and cmake_args[0] != '':
             cmake_args = []
             print('--cmake_args cannot be used with --ogs off! Ignoring!')
         if __format == 'singularity':
@@ -94,6 +91,13 @@ def main():  # pragma: no cover
             exit(0)
         info.make_dirs()
 
+        if ompi != 'off':
+            if args.base_image == 'ubuntu:20.04':
+                args.base_image = 'centos:8'
+                print(
+                    'Setting base_image to \'centos:8\'. OpenMPI is supported on CentOS only.'
+                )
+
         # Create definition
         hpccm.config.set_container_format(__format)
 
@@ -116,7 +120,10 @@ def main():  # pragma: no cover
                 if args.compiler == 'clang':
                     args.compiler_version = '8'
                 else:
-                    args.compiler_version = None  # Use default
+                    if hpccm.config.g_linux_distro == linux_distro.CENTOS:
+                        args.compiler_version = '10'  # required for std::filesystem
+                    else:
+                        args.compiler_version = None  # Use default
             if args.compiler == 'clang':
                 compiler = llvm(extra_repository=True,
                                 extra_tools=True,
@@ -146,7 +153,7 @@ def main():  # pragma: no cover
                 # Stage0 += ofed() OR mlnx_ofed(); is installed later on from debian archive
                 # Stage0 += knem()
                 Stage0 += ucx(version='1.5.1',
-                              cuda=False)  #  knem='/usr/local/knem'
+                              cuda=False)  # knem='/usr/local/knem'
                 Stage0 += packages(ospackages=['libpmi2-0-dev'
                                                ])  # req. for --with-pmi
                 # req. for --with-psm2
@@ -192,7 +199,7 @@ def main():  # pragma: no cover
                     ucx='/usr/local/ucx',
                     configure_opts=[
                         '--disable-getpwuid',
-                        '--sysconfdir=/mnt/0'
+                        '--sysconfdir=/mnt/0',
                         '--with-slurm',  # used on taurus
                         '--with-pmi=/usr/include/slurm-wlm',
                         'CPPFLAGS=\'-I /usr/include/slurm-wlm\'',
@@ -207,11 +214,12 @@ def main():  # pragma: no cover
                         '--enable-orterun-prefix-by-default',
                     ])
             else:
-                Stage0 += ucx(version='1.6.1' ,cuda=False)
+                ucx_version = '1.8.1'
+                Stage0 += ucx(version=ucx_version, cuda=False)
                 Stage0 += slurm_pmi2(version='17.02.11')
                 pmix_version = True
                 if version.parse(ompi) >= version.parse('4'):
-                    Stage0 += pmix()
+                    Stage0 += pmix(version='3.1.5')
                     pmix_version = '/usr/local/pmix'
 
                 mpicc = openmpi(version=ompi,
@@ -237,106 +245,183 @@ def main():  # pragma: no cover
                 })
 
             if args.mpi_benchmarks:
-                osu_app = scif(name='osu', file="_out/osu.scif")
-                osu_app += osu_benchmarks(toolchain=toolchain,
-                                          prefix='/scif/apps/osu')
-                Stage0 += osu_app
-                Stage0 += copy(src='ogscm/files/openmpi',
-                               dest='/usr/local/mpi-examples')
+                # osu_app = scif(name='osu', file=f"{info.out_dir}/osu.scif")
+                Stage0 += osu_benchmarks(toolchain=toolchain)
                 Stage0 += shell(commands=[
+                    'mkdir -p /usr/local/mpi-examples',
+                    'cd /usr/local/mpi-examples',
+                    'curl -O https://raw.githubusercontent.com/hpc/charliecloud/674b3b4e4ad243be5565f200d8f5fb92b7544480/examples/mpihello/hello.c',
+                    'curl -O https://computing.llnl.gov/tutorials/mpi/samples/C/mpi_bandwidth.c',
+                    'curl -O https://raw.githubusercontent.com/mpitutorial/mpitutorial/gh-pages/tutorials/mpi-send-and-receive/code/ring.c',
                     'mpicc -o /usr/local/bin/mpi-hello /usr/local/mpi-examples/hello.c',
                     'mpicc -o /usr/local/bin/mpi-ring /usr/local/mpi-examples/ring.c',
-                    'mpicc -o /usr/local/bin/mpi-bw /usr/local/mpi-examples/bw.c',
+                    'mpicc -o /usr/local/bin/mpi-bandwidth /usr/local/mpi-examples/mpi_bandwidth.c',
                 ])
                 Stage1 += copy(_from='build',
-                               src='/usr/local/bin/mpi_*',
+                               src='/usr/local/bin/mpi-*',
                                dest='/usr/local/bin/')
 
-        Stage0 += ogs_base()
+            # Stage0 += mlnx_ofed()
+
+        if ogs_version != 'clean':
+            Stage0 += ogs_base()
         if args.gui:
-            Stage0 += packages(ospackages=[
+            Stage0 += packages(apt=[
                 'mesa-common-dev', 'libgl1-mesa-dev', 'libglu1-mesa-dev',
                 'libxt-dev'
-            ])
-        if ogscm.config.g_package_manager == package_manager.CONAN:
-            Stage0 += cmake(eula=True, version='3.16.6')
-            conan_user_home = '/opt/conan'
-            if args.dev:
-                conan_user_home = ''
-            Stage0 += pm_conan(user_home=conan_user_home)
-            if not args.jenkins:
+            ],
+                               yum=[
+                                   'mesa-libOSMesa-devel', 'mesa-libGL-devel',
+                                   'mesa-libGLU-devel', 'libXt-devel'
+                               ])
+            Stage1 += packages(
+                apt=[
+                    'libosmesa6', 'libgl1-mesa-glx', 'libglu1-mesa', 'libxt6',
+                    'libopengl0'
+                ],
+                yum=['mesa-libOSMesa', 'mesa-libGL', 'mesa-libGLU', 'libXt'])
+        if ogs_version != 'clean':
+            if ogscm.config.g_package_manager == package_manager.CONAN:
+                Stage0 += cmake(eula=True, version='3.16.6')
+                conan_user_home = '/opt/conan'
+                if args.dev:
+                    conan_user_home = ''
+                Stage0 += pm_conan(user_home=conan_user_home)
                 Stage0 += environment(variables={'CONAN_SYSREQUIRES_SUDO': 0})
-        elif ogscm.config.g_package_manager == package_manager.SPACK:
-            vtk_variants = '+osmesa'
-            if ompi == 'off':
-                vtk_variants += ' -mpi'
-            spack_packages = [
-                # 'vtk@8.1.2' + vtk_variants,
-                'eigen@3.3.4',
-                'boost@1.68.0'
-            ]
-            Stage0 += pm_spack(
-                packages=spack_packages,
-                # ospackages=['libgl1-mesa-dev'],
-                repo='https://github.com/bilke/spack',
-                branch='patch-1')
-            Stage0 += shell(commands=[
-                '/opt/spack/bin/spack install --only dependencies vtk@8.1.2 +osmesa'
-            ])
-        elif ogscm.config.g_package_manager == package_manager.SYSTEM:
-            Stage0 += cmake(eula=True, version='3.16.6')
-            # Use ldconfig to set library search path (instead of
-            # LD_LIBRARY_PATH) as host var overwrites container var. See
-            # https://github.com/sylabs/singularity/pull/2669
-            # Stage0 += boost(version='1.66.0')  # header only?
-            Stage0 += packages(apt=['libboost-dev'], yum=['boost-devel'], epel=True)
-            # Stage0 += environment(variables={'BOOST_ROOT': '/usr/local/boost'})
-            Stage0 += eigen()
-            vtk_cmake_args = [
-                '-DModule_vtkIOXML=ON',
-                '-DVTK_Group_Rendering=OFF',
-                '-DVTK_Group_StandAlone=OFF'
-            ]
-            if args.gui:
-                Stage0 += packages(apt=[
-                        'libgeotiff-dev',
-                        'libshp-dev',
-                        'libqt5x11extras5-dev',
-                        'libqt5xmlpatterns5-dev',
+            elif ogscm.config.g_package_manager == package_manager.SYSTEM:
+                Stage0 += cmake(eula=True, version='3.16.6')
+                Stage0 += boost(version='1.66.0', bootstrap_opts=['headers'])
+                Stage0 += environment(
+                    variables={'BOOST_ROOT': '/usr/local/boost'})
+                vtk_cmake_args = [
+                    '-DModule_vtkIOXML=ON', '-DVTK_Group_Rendering=OFF',
+                    '-DVTK_Group_StandAlone=OFF'
+                ]
+                if args.gui:
+                    Stage0 += packages(apt=[
+                        'libgeotiff-dev', 'libshp-dev', 'libnetcdf-c++4-dev',
+                        'libqt5x11extras5-dev', 'libqt5xmlpatterns5-dev',
                         'qt5-default'
                     ],
-                    yum=[
-                        'libgeotiff-devel',
-                        'shapelib-devel',
-                        'qt5-qtbase',
-                        'qt5-qtxmlpatterns',
-                        'qt5-qtx11extras'
+                                       yum=[
+                                           'libgeotiff-devel',
+                                           'shapelib-devel', 'netcdf-devel',
+                                           'qt5-qtbase-devel',
+                                           'qt5-qtxmlpatterns-devel',
+                                           'qt5-qtx11extras-devel'
+                                       ])
+                    Stage1 += packages(apt=[
+                        'geotiff-bin', 'shapelib', 'libnetcdf-c++4',
+                        'libqt5x11extras5', 'libqt5xmlpatterns5', 'qt5-default'
+                    ],
+                                       yum=[
+                                           'libgeotiff', 'shapelib', 'netcdf',
+                                           'qt5-qtbase', 'qt5-qtxmlpatterns',
+                                           'qt5-qtx11extras'
+                                       ])
+                    vtk_cmake_args = [
+                        '-DVTK_BUILD_QT_DESIGNER_PLUGIN=OFF',
+                        '-DVTK_Group_Qt=ON', '-DVTK_QT_VERSION=5'
+                    ]
+                if hpccm.config.g_linux_distro == linux_distro.CENTOS:
+                    # otherwise linker error, maybe due to gcc 10?
+                    vtk_cmake_args.extend([
+                        '-DBUILD_SHARED_LIBS=OFF',
+                        '-DCMAKE_POSITION_INDEPENDENT_CODE=ON'
                     ])
-                vtk_cmake_args = [
-                    '-DVTK_BUILD_QT_DESIGNER_PLUGIN=OFF',
-                    '-DVTK_Group_Qt=ON',
-                    '-DVTK_QT_VERSION=5'
-                ]
-            if args.insitu:
-                if args.gui:
-                    print('--gui can not be used with --insitu!')
-                    exit(1)
-                Stage0 += paraview(cmake_args=['-D PARAVIEW_USE_PYTHON=ON'],
-                                   edition='CATALYST',
-                                   ldconfig=True,
-                                   toolchain=toolchain)
-            else:
-                Stage0 += vtk(cmake_args=vtk_cmake_args,
-                              toolchain=toolchain,
-                              ldconfig=True)
-            if ompi != 'off':
-                Stage0 += petsc(version='3.11.3', ldconfig=True)
+                if args.insitu:
+                    if args.gui:
+                        print('--gui can not be used with --insitu!')
+                        exit(1)
+                    Stage0 += paraview(cmake_args=['-DPARAVIEW_USE_PYTHON=ON'],
+                                       edition='CATALYST',
+                                       ldconfig=True,
+                                       toolchain=toolchain,
+                                       version="v5.8.1")
+                else:
+                    if toolchain.CC == 'mpicc':
+                        vtk_cmake_args.extend([
+                            '-D Module_vtkIOParallelXML=ON',
+                            '-D Module_vtkParallelMPI=ON'
+                        ])
+                    Stage0 += generic_cmake(
+                        cmake_opts=vtk_cmake_args,
+                        devel_environment={'VTK_ROOT': '/usr/local/vtk'},
+                        directory='VTK-8.2.0',
+                        ldconfig=True,
+                        prefix='/usr/local/vtk',
+                        toolchain=toolchain,
+                        url=
+                        'https://www.vtk.org/files/release/8.2/VTK-8.2.0.tar.gz'
+                    )
+                if ompi != 'off':
+                    Stage0 += packages(yum=['diffutils'])
+                    Stage0 += generic_autotools(
+                        configure_opts=[
+                            f'CC={toolchain.CC}', f'CXX={toolchain.CXX}',
+                            '--CFLAGS=\'-O3\'', '--CXXFLAGS=\'-O3\'',
+                            '--FFLAGS=\'-O3\'', '--with-debugging=no',
+                            '--with-fc=0', '--download-f2cblaslapack=1'
+                        ],
+                        devel_environment={'PETSC_DIR': '/usr/local/petsc'},
+                        directory='petsc-3.11.3',
+                        ldconfig=True,
+                        preconfigure=[
+                            "sed -i -- 's/python/python3/g' configure"
+                        ],
+                        prefix='/usr/local/petsc',
+                        toolchain=toolchain,
+                        url='http://ftp.mcs.anl.gov/pub/petsc/release-snapshots/'
+                        'petsc-lite-3.11.3.tar.gz')
+
+                Stage0 += generic_cmake(
+                    devel_environment={
+                        'Eigen3_ROOT': '/usr/local/eigen',
+                        'Eigen3_DIR': '/usr/local/eigen'
+                    },
+                    directory='eigen-3.3.7',
+                    prefix='/usr/local/eigen',
+                    url=
+                    'https://gitlab.com/libeigen/eigen/-/archive/3.3.7/eigen-3.3.7.tar.gz'
+                )
         if args.cvode:
-            Stage0 += cvode()
+            Stage0 += generic_cmake(
+                cmake_opts=[
+                    '-D EXAMPLES_INSTALL=OFF', '-D BUILD_SHARED_LIBS=OFF',
+                    '-D CMAKE_POSITION_INDEPENDENT_CODE=ON'
+                ],
+                devel_environment={'CVODE_ROOT': '/usr/local/cvode'},
+                directory='cvode-2.8.2',
+                prefix='/usr/local/cvode',
+                url='https://github.com/ufz/cvode/archive/2.8.2.tar.gz')
+
         if args.cppcheck:
-            Stage0 += cppcheck()
+            Stage0 += generic_cmake(
+                devel_environment={'PATH': '/usr/local/cppcheck/bin:$PATH'},
+                directory='cppcheck-809a769c690d8ab6fef293e41a29c8490512866e',
+                prefix='/usr/local/cppcheck',
+                runtime_environment={'PATH': '/usr/local/cppcheck/bin:$PATH'},
+                url=
+                'https://github.com/danmar/cppcheck/archive/809a769c690d8ab6fef293e41a29c8490512866e.tar.gz'
+            )
+
         if args.iwyy and args.compiler == 'clang':
-            Stage0 += iwyy(clang_version=args.compiler_version)
+            Stage0 += packages(ospackages=[
+                'libncurses5-dev', 'zlib1g-dev',
+                f"llvm-{args.compiler_version}-dev",
+                f"libclang-{args.compiler_version}-dev"
+            ])
+            Stage0 += generic_cmake(
+                cmake_opts=[
+                    f"-D IWYU_LLVM_ROOT_PATH=/usr/lib/llvm-{args.compiler_version}"
+                ],
+                devel_environment={'PATH': '/usr/local/iwyy/bin:$PATH'},
+                directory=
+                f"include-what-you-use-clang_{args.compiler_version}.0",
+                prefix='/usr/local/iwyy',
+                runtime_environment={'PATH': '/usr/local/iwyy/bin:$PATH'},
+                url="https://github.com/include-what-you-use/include-what-"
+                f"you-use/archive/clang_{args.compiler_version}.0.tar.gz")
         if args.docs:
             Stage0 += packages(
                 ospackages=['doxygen', 'graphviz', 'texlive-base'])
@@ -355,11 +440,20 @@ def main():  # pragma: no cover
         if args.packages:
             Stage0 += packages(ospackages=args.packages)
 
+        if args.tfel:
+            Stage0 += generic_cmake(
+                directory='tfel-TFEL-3.3.0',
+                ldconfig=True,
+                url='https://github.com/thelfer/tfel/archive/TFEL-3.3.0.tar.gz',
+                prefix='/usr/local/tfel')
+            Stage0 += environment(variables={'TFELHOME': '/usr/local/tfel'})
+
         definition_file_path = os.path.join(info.out_dir, info.definition_file)
-        if ogs_version != 'off':
+        if args.ccache:
+            Stage0 += ccache(cache_size='15G')
+        if ogs_version != 'off' and ogs_version != 'clean':
             mount_args = ''
             if args.ccache:
-                Stage0 += ccache(cache_size='15G')
                 mount_args = f'{mount_args} --mount=type=cache,target=/opt/ccache,id=ccache'
             if args.cvode:
                 cmake_args.append('-DOGS_USE_CVODE=ON')
@@ -388,19 +482,14 @@ def main():  # pragma: no cover
                            prefix='/scif/apps/ogs',
                            cmake_args=cmake_args,
                            parallel=args.parallel,
-                           skip_lfs=True,
                            remove_build=True,
                            remove_source=True)
             Stage0 += ogs_app
 
-        if args.jenkins:
-            Stage0 += ccache(cache_size='15G')
-            Stage0 += jenkins_node()
-
         stages_string = str(Stage0)
 
         if args.runtime_only:
-            Stage1 += Stage0.runtime()
+            Stage1 += Stage0.runtime(exclude=['boost'])
             if args.compiler == 'gcc' and args.compiler_version != None:
                 Stage1 += packages(apt=['libstdc++6'])
             stages_string += "\n\n" + str(Stage1)
@@ -411,7 +500,8 @@ def main():  # pragma: no cover
         if args.print:
             print(stages_string)
         else:
-            print(f'Created definition {os.path.abspath(definition_file_path)}')
+            print(
+                f'Created definition {os.path.abspath(definition_file_path)}')
 
         # Create image
         if not args.build:
@@ -428,7 +518,7 @@ def main():  # pragma: no cover
             # TODO: adapt this to else
             continue
 
-        build_cmd = (f"DOCKER_BUILDKIT=1 docker build "
+        build_cmd = (f"DOCKER_BUILDKIT=1 docker build {args.build_args} "
                      f"-t {info.tag} -f {definition_file_path} .")
         print(f"Running: {build_cmd}")
         subprocess.run(build_cmd, shell=True)
